#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sqlite3.h>
#include <unistd.h>
#include <sys/stat.h>
#include <pwd.h>
#include <sys/types.h>
#include <stdexcept>

sqlite3* _sqlite3 = nullptr;

bool directory_exists(const char* file_path)	
{
	struct stat ss;
	if (stat(file_path, &ss) != 0)
		return false;
	else
		return (ss.st_mode & S_IFMT) == S_IFDIR;
}

bool file_exists(const char* file_path)	
{
	struct stat ss;
	if (stat(file_path, &ss) != 0)
		return false;
	else
		return (ss.st_mode & S_IFMT) == S_IFREG;
}

std::string get_cesta_db()
{
	std::string db_file = getenv("HOME");
	if (!directory_exists(db_file.c_str()))
		db_file = getpwuid(getuid())->pw_dir;
	if (!directory_exists(db_file.c_str()))
		throw std::runtime_error("Nepodařilo se zjistit domovský adresář.");
	db_file.append("/Dokumenty/images.sqlite");
	return db_file;
}

void vytvorit_databazi()
{
	std::string str_file = get_cesta_db();
	if (file_exists(str_file.c_str()))
	{
		if (remove(str_file.c_str()) !=0)
			throw std::runtime_error("Chyba smazání souboru");
	}
	int ires = sqlite3_open(str_file.c_str(), &_sqlite3);
	if (ires != SQLITE_OK)
		throw std::runtime_error(sqlite3_errstr(ires));
	ires = sqlite3_exec(_sqlite3, "CREATE TABLE IF NOT EXISTS polozky (id INTEGER PRIMARY KEY,"
		" nazev TEXT, obrazek BLOB)", nullptr, nullptr, nullptr);
	if (ires != SQLITE_OK)
		throw std::runtime_error(sqlite3_errstr(ires));
}

void pridat_obrazek(const char* file_path)
{
	struct stat stat_file;
	if (stat(file_path, &stat_file) != 0)
		throw std::runtime_error("chyba vstupního souboru");
	if ((stat_file.st_mode & S_IFMT) != S_IFREG)
		throw std::runtime_error("vstupního soubor neexistuje");
	if (0 == stat_file.st_size)
		throw std::runtime_error("soubor je prázdný");
	FILE* pfile = nullptr;
	void* file_buffer = nullptr;
	try
	{
		pfile = fopen(file_path, "r");
		if (nullptr == pfile)
			throw std::runtime_error("chyba fopen");
		file_buffer = malloc(stat_file.st_size);
		if (nullptr == file_buffer)
			throw std::runtime_error("chyba alokace - malloc");
		if (fread(file_buffer, stat_file.st_size, 1, pfile) != 1)
			throw std::runtime_error("chyba čtení souboru");
		sqlite3_stmt* stmt = nullptr;
		char sz_sql[255];
		strcpy(sz_sql, "INSERT INTO polozky (nazev, obrazek) VALUES (?,?)");
		int sqlres = sqlite3_prepare_v2(_sqlite3, sz_sql,
			static_cast<int>(strlen(sz_sql)), &stmt, nullptr);
		if (sqlres != SQLITE_OK)
			throw std::runtime_error(sqlite3_errstr(sqlres));
		sqlres = sqlite3_bind_text(stmt, 1, file_path, (int)(strlen(file_path)), SQLITE_STATIC);
		if (sqlres != SQLITE_OK)
			throw std::runtime_error(sqlite3_errstr(sqlres));
		sqlres = sqlite3_bind_blob64(stmt, 2, (const void*)file_buffer,
				(sqlite3_int64)stat_file.st_size, 0);
		if (sqlres != SQLITE_OK)
			throw std::runtime_error(sqlite3_errstr(sqlres));
		sqlres = sqlite3_step(stmt);
		if ((sqlres != SQLITE_OK) && (sqlres != SQLITE_DONE))
			throw std::runtime_error(sqlite3_errstr(sqlres));
		sqlres = sqlite3_reset(stmt);
		if ((sqlres != SQLITE_OK) && (sqlres != SQLITE_DONE))
			throw std::runtime_error(sqlite3_errstr(sqlres));
		sqlres = sqlite3_finalize(stmt);
		if ((sqlres != SQLITE_OK) && (sqlres != SQLITE_DONE))
			throw std::runtime_error(sqlite3_errstr(sqlres));
		fclose(pfile);
		free(file_buffer);
	}
	catch(const std::exception& e)
	{
		if (file_buffer)
			free(file_buffer);
		if (pfile)
			fclose(pfile);
		throw e;
	}
}

void ulozit_obrazky()
{
	std::string str_file = get_cesta_db();
	if (!file_exists(str_file.c_str()))
		throw std::runtime_error("soubor databáze neexistuje");
	int sqlres = sqlite3_open(str_file.c_str(), &_sqlite3);
	if (sqlres != SQLITE_OK)
		throw std::runtime_error(sqlite3_errstr(sqlres));
	sqlite3_stmt* stmt;
	const char* strsql = "SELECT nazev, obrazek FROM polozky";
	sqlres = sqlite3_prepare_v2(_sqlite3, strsql,
		static_cast<int>(strlen(strsql)), &stmt, nullptr);
	if (sqlres != SQLITE_OK)
		throw std::runtime_error(sqlite3_errstr(sqlres));
	int istep;
	size_t sizedata;
	void* pdata;
	istep = sqlite3_step(stmt);
	FILE* pfile;
	while (SQLITE_ROW == istep)
	{
		sizedata = sqlite3_column_bytes(stmt, 1);
		pdata = (void*)sqlite3_column_blob(stmt, 1);
		pfile = fopen((const char*)sqlite3_column_text(stmt, 0), "w");
		if (nullptr == pfile)
		{
			istep = sqlite3_step(stmt);
			continue;
		}
		if (1 != fwrite((const void*)pdata, sizedata, 1, pfile))
		{
			printf("chyba zápisu do: %s\n", sqlite3_column_text(stmt, 1));
		}
		fclose(pfile);
		istep = sqlite3_step(stmt);
	}
	sqlres = sqlite3_reset(stmt);
	if (sqlres != SQLITE_OK)
		throw std::runtime_error(sqlite3_errstr(sqlres));
	sqlres = sqlite3_finalize(stmt);
	if (sqlres != SQLITE_OK)
		throw std::runtime_error(sqlite3_errstr(sqlres));
	sqlite3_close(_sqlite3);
}

int main(int argc, char const *argv[])
{
	printf("start...\n");
	try
	{
/*	vytvorit_databazi();
	pridat_obrazek("/home/radek/Obrázky/pozadi-pivo.jpg");
	pridat_obrazek("/home/radek/Obrázky/jana.jpg");
	pridat_obrazek("/home/radek/Obrázky/eso.bmp");
	sqlite3_close(_sqlite3);*/
		ulozit_obrazky();
	}
	catch(const std::exception& e)
	{
		printf("Došlo k výjimce: %s\n", e.what());
		return EXIT_FAILURE;
	}
	printf("hotovo\n");
	return EXIT_SUCCESS;
}
